<?php

/* Add menu */

add_action( 'admin_menu', 'camper_admin_add_pages' );

function camper_admin_add_pages() {
	global $current_user;

	if ( isset( $_POST['camper-save-event'] ) ) {
		$id = (int) $_POST['event'];
		check_admin_referer( 'camper-save-event-' . $id );

		if ( -1 == $id || ! ( $event = camper_event( $id ) ) ) {
			$event = new Camper_Event();
			$event->owner_id = $current_user->ID;
			$event->created_at = current_time( 'mysql' );
		}

		$query = array();
		$query['message'] = ( $event->initial ) ? 'created' : 'saved';

		if ( ! $name = trim( $_POST['name'] ) )
			$name = __( 'No-name event', 'camper' );

		$event->name = $name;

		$options = array();
		$options['confirmation_notice'] = trim( $_POST['confirmation-notice'] );
		$options['confirmation_notice_subject'] = trim( $_POST['confirmation-notice-subject'] );
		$options['confirmation_notice_body'] = trim( $_POST['confirmation-notice-body'] );

		$event->options = $options;

		$event->save();

		foreach ( (array) $_POST['items'] as $item_id ) {
			$item_id = (int) $item_id;
			$item_name = trim( $_POST['item_name'][$item_id] );
			$item_price = $_POST['item_price'][$item_id];
			$item_currency = $_POST['item_currency'][$item_id];

			if ( 0 < $item_id && $item = camper_item( $item_id ) ) {
				if ( $_POST['item_delete'][$item_id] ) {
					$item->delete();
				} else {
					$item->name = $item_name;
					$item->price = $item_price;
					$item->currency = $item_currency;
					$item->save();
				}
			} elseif ( $item_name ) {
				$event->add_item(
					trim( $_POST['item_name'][$item_id] ),
					$_POST['item_price'][$item_id],
					$_POST['item_currency'][$item_id] );
			}
		}

		if ( ! $event->items() )
			$event->add_item( $name );

		$query['event'] = $event->id;
		$redirect_to = camper_admin_url( $query );
		wp_redirect( $redirect_to );
		exit();

	} elseif ( isset( $_POST['camper-delete-event'] ) ) {
		$id = (int) $_POST['event'];
		check_admin_referer( 'camper-delete-event-' . $id );

		if ( $event = camper_event( $id ) )
			$event->delete();

		$redirect_to = camper_admin_url( array( 'message' => 'deleted' ) );
		wp_redirect( $redirect_to );
		exit();
	} elseif ( ( isset( $_GET['camper-bulk-edit-attendees'] )
			|| isset( $_GET['camper-bulk-edit-attendees2'] ) )
			&& $event = camper_event( $_GET['event'] ) ) {
		check_admin_referer( 'camper-bulk-edit-attendees' );

		$action = isset( $_GET['camper-bulk-edit-attendees'] )
			? $_GET['bulkaction'] : $_GET['bulkaction2'];
		$attendees = (array) stripslashes_deep( $_GET['attendee'] );

		$message = '';

		foreach ( $attendees as $attendee_id ) {
			if ( 'confirm' == $action ) {
				if ( $event->confirm( absint( $attendee_id ) ) && '' == $message )
					$message = 'attendees_confirmed';
			} elseif ( 'unconfirm' == $action ) {
				if ( $event->unconfirm( absint( $attendee_id ) ) && '' == $message )
					$message = 'attendees_unconfirmed';
			}
		}

		$redirect_to = camper_admin_url( array(
			'action' => 'attendees', 'event' => $event->id, 'message' => $message ) );
		wp_redirect( $redirect_to );
		exit();
	}

	add_object_page( __( 'Events', 'camper' ), __( 'Camper', 'camper' ),
		CAMPER_ADMIN_CAPABILITY, 'camper', 'camper_admin_events' );

	add_submenu_page( 'camper', __( 'Events', 'camper' ), __( 'Events', 'camper' ),
		CAMPER_ADMIN_CAPABILITY, 'camper', 'camper_admin_events' );
}

function camper_admin_events() {
	global $wpdb, $camper;

	$events = $wpdb->get_results( "SELECT id, name FROM $camper->events" );

	if ( ! $events || 'new' == $_GET['event'] ) {
		$current = -1;
	} elseif ( $event = camper_event( $_GET['event'] ) ) {
		$current = (int) $_GET['event'];
	} else {
		$first = reset( $events ); // Returns first item
		$current = $first->id;
		$event = camper_event( $current );
	}

	if ( 'attendees' == $_GET['action'] && $event ) {
		if ( ! $pagenum = isset( $_GET['pagenum'] ) ? absint( $_GET['pagenum'] ) : 0 )
			$pagenum = 1;

		$perpage = 20;
		$pageoffset = ( $pagenum - 1 ) * $perpage;

		$search = stripslashes( $_GET['s'] );

		$args = array(
			'perpage' => $perpage,
			'offset' => $pageoffset,
			'search' => $search );

		if ( $showitem = absint( $_GET['showitem'] ) )
			$args['showitem'] = $showitem;

		if ( ( $showstatus = trim( $_GET['showstatus'] ) ) && 'confirmed_only' == $showstatus )
			$args['confirmed_only'] = 1;

		$items = $event->items();
		$attendees = $event->attendees( $args );

		require_once CAMPER_PLUGIN_DIR . '/admin/attendees.php';
		return;
	}

	require_once CAMPER_PLUGIN_DIR . '/admin/events.php';
}

add_action( 'admin_print_scripts', 'camper_admin_enqueue_scripts' );

function camper_admin_enqueue_scripts() {
	global $plugin_page;

	if ( 'camper' == $plugin_page ) {
		wp_enqueue_script( 'camper', camper_plugin_url( 'admin/scripts.js' ),
			array( 'jquery' ), CAMPER_VERSION, true );
	}
}

add_action( 'admin_print_styles', 'camper_admin_enqueue_styles' );

function camper_admin_enqueue_styles() {
	global $plugin_page;

	if ( 'camper' == $plugin_page ) {
		wp_enqueue_style( 'camper',
			camper_plugin_url( 'admin/styles.css' ),
			array(), CAMPER_VERSION, 'all' );
	}
}

add_action( 'camper_admin_events_after_title', 'camper_update_message' );
add_action( 'camper_admin_attendees_after_title', 'camper_update_message' );

function camper_update_message( &$event ) {
	switch ( $_GET['message'] ) {
		case 'created':
			$message = __( "Event created.", 'camper' );
			break;
		case 'saved':
			$message = __( "Event saved.", 'camper' );
			break;
		case 'deleted':
			$message = __( "Event deleted.", 'camper' );
			break;
		case 'attendees_confirmed':
			$message = __( "Attendees confirmed.", 'camper' );
			break;
		case 'attendees_unconfirmed':
			$message = __( "Attendees unconfirmed.", 'camper' );
			break;
	}

	if ( ! $message )
		return;

?>
<div id="message" class="updated fade"><p><?php echo esc_html( $message ); ?></p></div>
<?php
}

?>