<?php

/* Class: Camper_Event */

class Camper_Event {

	var $initial = true;

	function items( $args = '' ) {
		global $wpdb, $camper;

		$query = $wpdb->prepare(
			"SELECT * FROM $camper->items WHERE event_id = %d",
			absint( $this->id ) );

		if ( $items = $wpdb->get_results( $query ) )
			return camper_item( $items );
	}

	function add_item( $name, $price = 0, $currency = 'USD' ) {
		$item = new Camper_Item();
		$item->event_id = $this->id;
		$item->name = $name;
		$item->price = $price;
		$item->currency = $currency;

		if ( $item->save() )
			return $item;

		return false;
	}

	function attendees( $args = '' ) {
		global $wpdb, $camper;

		$defaults = array(
			'perpage' => 0,
			'offset' => 0,
			'showitem' => 0,
			'confirmed_only' => 0,
			'user_id' => 0,
			'search' => '' );

		$args = wp_parse_args( $args, $defaults );
		extract( $args, EXTR_SKIP );

		$query = "SELECT * FROM $camper->attendees as attendees INNER JOIN $wpdb->users as users ON (attendees.user_id = users.ID) WHERE 1=1";

		$query .= $wpdb->prepare( " AND event_id = %d", absint( $this->id ) );

		if ( $showitem )
			$query .= $wpdb->prepare( " AND item_id = %d", $showitem );

		if ( $confirmed_only )
			$query .= " AND confirmed = 1";

		if ( $user_id )
			$query .= $wpdb->prepare( " AND user_id = %d", absint( $user_id ) );

		if ( $search = trim( stripslashes( $search ) ) ) {
			$search = explode( ' ', $search );
			foreach( $search as $s ) {
				$query .= " AND (1 = 0";
				$query .= $wpdb->prepare( " OR attendees.payment_hash LIKE %s", '%' . $s . '%' );
				$query .= $wpdb->prepare( " OR users.user_login LIKE %s", '%' . $s . '%' );
				$query .= $wpdb->prepare( " OR users.user_email LIKE %s", '%' . $s . '%' );
				$query .= $wpdb->prepare( " OR users.display_name LIKE %s", '%' . $s . '%' );
				$query .= ")";
			}
		}

		if ( $offset && $perpage ) {
			$query .= $wpdb->prepare( " LIMIT %d,%d", $offset, $perpage );
		} elseif ( $perpage ) {
			$query .= $wpdb->prepare( " LIMIT %d", $perpage );
		}

		if ( $attendees = $wpdb->get_results( $query ) )
			return camper_attendee( $attendees );
	}

	function add_attendee( $item_id, $user_id ) {
		if ( ! ( $item = camper_item( $item_id ) )
		|| ( $item->event_id != $this->id )
		|| ! $user_id )
			return false;

		$attendee = new Camper_Attendee();
		$attendee->event_id = $this->id;
		$attendee->item_id = $item->id;
		$attendee->user_id = $user_id;
		$attendee->submitted_at = current_time( 'mysql' );
		$attendee->payment_hash = camper_get_payment_hash();

		$free = ! $item->price;

		if ( $free ) {
			$attendee->confirmed = 1;
			$attendee->confirmed_at = current_time( 'mysql' );
		}

		if ( $attendee->save() ) {
			if ( $free && $this->option( 'confirmation_notice' ) )
				$this->notice_confirmation( $attendee->id );

			return $attendee;
		}

		return false;
	}

	function confirm( $attendee_id ) {
		if ( ! $attendee = camper_attendee( $attendee_id ) )
			return false;

		if ( $attendee->confirmed ) // Already confirmed
			return;

		$attendee->confirmed = 1;
		$attendee->confirmed_at = current_time( 'mysql' );

		if ( ! $attendee->save() )
			return false;

		if ( $this->option( 'confirmation_notice' ) )
			$this->notice_confirmation( $attendee_id );

		return true;
	}

	function unconfirm( $attendee_id ) {
		if ( ! $attendee = camper_attendee( $attendee_id ) )
			return false;

		if ( ! $attendee->confirmed ) // Already unconfirmed
			return;

		$attendee->confirmed = 0;
		$attendee->confirmed_at = '0000-00-00 00:00:00';

		if ( ! $attendee->save() )
			return false;

		return true;
	}

	function notice_confirmation( $attendee_id ) {
		$attendee = camper_attendee( $attendee_id );
		$attendee_user = get_userdata( $attendee->user_id );

		$event_owner = get_userdata( $this->owner_id );

		$sender = $event_owner->user_email;

		if ( ! $subject = $this->option( 'confirmation_notice_subject' ) )
			$subject = $this->name;

		$subject = preg_replace(
			array( '/%user\.display_name%/', '%user\.email%' ),
			array( $attendee_user->display_name, $attendee_user->user_email ),
			$subject );

		if ( ! $body = $this->option( 'confirmation_notice_body' ) )
			$body = __( "Your application is confirmed.", 'camper' );

		$body = preg_replace(
			array( '/%user\.display_name%/', '%user\.email%' ),
			array( $attendee_user->display_name, $attendee_user->user_email ),
			$body );

		$recipient = $attendee_user->user_email;

		$headers = "From: $sender\n";

		return @wp_mail( $recipient, $subject, $body, $headers );
	}

	function option( $name ) {
		if ( ! $name = trim( $name ) )
			return;

		$options = $this->options;

		if ( is_array( $options ) )
			return $options[$name];
	}

	function save() {
		global $wpdb, $camper;

		$data = array(
			'owner_id' => $this->owner_id,
			'name' => stripslashes_deep( $this->name ),
			'closed' => $this->closed,
			'closed_at' => $this->closed_at,
			'created_at' => $this->created_at,
			'options' => maybe_serialize( stripslashes_deep( $this->options ) ) );

		$format = array( '%d', '%s', '%d', '%s', '%s', '%s' );

		if ( $this->initial ) {
			$result = $wpdb->insert( $camper->events, $data, $format );

			if ( $result ) {
				$this->initial = false;
				$this->id = $wpdb->insert_id;

				do_action_ref_array( 'camper_after_create_event', array( &$this ) );
			} else {
				return false; // Failed to save
			}

		} else { // Update
			if ( ! (int) $this->id )
				return false; // Missing ID

			$result = $wpdb->update( $camper->events, $data, array( 'id' => $this->id ),
				$format, '%d' );

			if ( false !== $result ) {
				do_action_ref_array( 'camper_after_update_event', array( &$this ) );
			} else {
				return false; // Failed to save
			}
		}

		do_action_ref_array( 'camper_after_save_event', array( &$this ) );
		return true; // Succeeded to save
	}

	function delete() {
		global $wpdb, $camper;

		if ( $this->initial )
			return;

		$this->delete_items();
		$this->delete_attendees();

		$query = $wpdb->prepare(
			"DELETE FROM $camper->events WHERE id = %d LIMIT 1",
			absint( $this->id ) );

		$wpdb->query( $query );

		$this->initial = true;
		$this->id = null;
	}

	function delete_items() {
		global $wpdb, $camper;

		if ( $this->initial )
			return;

		$query = $wpdb->prepare(
			"DELETE FROM $camper->items WHERE event_id = %d",
			absint( $this->id ) );

		$wpdb->query( $query );
	}

	function delete_attendees() {
		global $wpdb, $camper;

		if ( $this->initial )
			return;

		$query = $wpdb->prepare(
			"DELETE FROM $camper->attendees WHERE event_id = %d",
			absint( $this->id ) );

		$wpdb->query( $query );
	}
}

function camper_event( $data ) {
	global $wpdb, $camper;

	if ( is_array( $data ) ) { // Maybe an array of objects.
		$events = array();

		foreach ( $data as $obj ) {
			if ( $event = camper_event( $obj ) )
				$events[] = $event;
		}

		return $events;
	}

	if ( is_string( $data ) )
		$data = (int) $data;

	if ( is_int( $data ) ) { // $data is id.
		$id = (int) $data;

		$query = $wpdb->prepare( "SELECT * FROM $camper->events WHERE id = %d", $id );

		if ( ! $data = $wpdb->get_row( $query ) )
			return false; // No data
	}

	if ( is_object( $data ) ) {
		$event = new Camper_Event();
		$event->initial = false;
		$event->id = $data->id;
		$event->owner_id = $data->owner_id;
		$event->name = $data->name;
		$event->closed = $data->closed;
		$event->closed_at = $data->closed_at;
		$event->created_at = $data->created_at;
		$event->options = maybe_unserialize( $data->options );

		$query = $wpdb->prepare(
			"SELECT COUNT(id) FROM $camper->attendees WHERE event_id = %d",
			$data->id );

		$event->attendees_count = (int) $wpdb->get_var( $query );

		return $event;
	}

	return false;
}

function camper_get_payment_hash( $char_length = 4 ) {
	global $wpdb, $camper;

	$chars = 'ABCDEFGHJKLMNPQRSTUVWXYZ23456789';

	$payment_hash = '';
	for ( $i = 0; $i < $char_length; $i++ ) {
		$pos = mt_rand( 0, strlen( $chars ) - 1 );
		$char = $chars[$pos];
		$payment_hash .= $char;
	}

	$query = $wpdb->prepare(
		"SELECT id FROM $camper->attendees WHERE payment_hash LIKE %s",
		$payment_hash );

	if ( $wpdb->get_var( $query ) ) // Not unique
		return camper_get_payment_hash();

	return apply_filters( 'camper_get_payment_hash', $payment_hash );
}


/* Class: Camper_Item */

class Camper_Item {

	var $initial = true;

	function save() {
		global $wpdb, $camper;

		$data = array(
			'event_id' => $this->event_id,
			'name' => stripslashes_deep( $this->name ),
			'price' => $this->price,
			'currency' => $this->currency );

		$format = array( '%d', '%s', '%s', '%s' );

		if ( $this->initial ) {
			$result = $wpdb->insert( $camper->items, $data, $format );

			if ( $result ) {
				$this->initial = false;
				$this->id = $wpdb->insert_id;

				do_action_ref_array( 'camper_after_create_item', array( &$this ) );
			} else {
				return false; // Failed to save
			}

		} else { // Update
			if ( ! (int) $this->id )
				return false; // Missing ID

			$result = $wpdb->update( $camper->items, $data, array( 'id' => $this->id ),
				$format, '%d' );

			if ( false !== $result ) {
				do_action_ref_array( 'camper_after_update_item', array( &$this ) );
			} else {
				return false; // Failed to save
			}
		}

		do_action_ref_array( 'camper_after_save_item', array( &$this ) );
		return true; // Succeeded to save
	}

	function delete() {
		global $wpdb, $camper;

		if ( $this->initial )
			return;

		$query = $wpdb->prepare(
			"DELETE FROM $camper->items WHERE id = %d LIMIT 1",
			absint( $this->id ) );

		$wpdb->query( $query );

		$this->initial = true;
		$this->id = null;
	}
}

function camper_item( $data ) {
	global $wpdb, $camper;

	if ( is_array( $data ) ) { // Maybe an array of objects.
		$items = array();

		foreach ( $data as $obj ) {
			if ( $item = camper_item( $obj ) )
				$items[] = $item;
		}

		return $items;
	}

	if ( is_string( $data ) )
		$data = (int) $data;

	if ( is_int( $data ) ) { // $data is id.
		$id = absint( $data );

		$query = $wpdb->prepare( "SELECT * FROM $camper->items WHERE id = %d", $id );

		if ( ! $data = $wpdb->get_row( $query ) )
			return false; // No data
	}

	if ( is_object( $data ) ) {
		$item = new Camper_Item();
		$item->initial = false;
		$item->id = $data->id;
		$item->event_id = $data->event_id;
		$item->name = $data->name;
		$item->price = $data->price;
		$item->currency = $data->currency;

		$query = $wpdb->prepare(
			"SELECT COUNT(id) FROM $camper->attendees WHERE event_id = %d AND item_id = %d",
			$data->event_id, $data->id );

		$item->attendees_count = (int) $wpdb->get_var( $query );

		return $item;
	}

	return false;
}


/* Class: Camper_Attendee */

class Camper_Attendee {

	var $initial = true;

	function save() {
		global $wpdb, $camper;

		$data = array(
			'event_id' => $this->event_id,
			'item_id' => $this->item_id,
			'user_id' => $this->user_id,
			'submitted_at' => $this->submitted_at,
			'payment_hash' => stripslashes_deep( $this->payment_hash ),
			'confirmed' => $this->confirmed,
			'confirmed_at' => $this->confirmed_at );

		$format = array( '%d', '%d', '%d', '%s', '%s', '%d', '%s' );

		if ( $this->initial ) {
			$result = $wpdb->insert( $camper->attendees, $data, $format );

			if ( $result ) {
				$this->initial = false;
				$this->id = $wpdb->insert_id;

				do_action_ref_array( 'camper_after_create_attendee', array( &$this ) );
			} else {
				return false; // Failed to save
			}

		} else { // Update
			if ( ! (int) $this->id )
				return false; // Missing ID

			$result = $wpdb->update( $camper->attendees, $data, array( 'id' => $this->id ),
				$format, '%d' );

			if ( false !== $result ) {
				do_action_ref_array( 'camper_after_update_attendee', array( &$this ) );
			} else {
				return false; // Failed to save
			}
		}

		do_action_ref_array( 'camper_after_save_attendee', array( &$this ) );
		return true; // Succeeded to save
	}

	function delete() {
		global $wpdb, $camper;

		if ( $this->initial )
			return;

		$query = $wpdb->prepare(
			"DELETE FROM $camper->attendees WHERE id = %d LIMIT 1",
			absint( $this->id ) );

		$wpdb->query( $query );

		$this->initial = true;
		$this->id = null;
	}
}

function camper_attendee( $data ) {
	global $wpdb, $camper;

	if ( is_array( $data ) ) { // Maybe an array of objects.
		$attendees = array();

		foreach ( $data as $obj ) {
			if ( $attendee = camper_attendee( $obj ) )
				$attendees[] = $attendee;
		}

		return $attendees;
	}

	if ( is_string( $data ) )
		$data = (int) $data;

	if ( is_int( $data ) ) { // $data is id.
		$id = absint( $data );

		$query = $wpdb->prepare( "SELECT * FROM $camper->attendees WHERE id = %d", $id );

		if ( ! $data = $wpdb->get_row( $query ) )
			return false; // No data
	}

	if ( is_object( $data ) ) {
		$attendee = new Camper_Attendee();
		$attendee->initial = false;
		$attendee->id = $data->id;
		$attendee->event_id = $data->event_id;
		$attendee->item_id = $data->item_id;
		$attendee->user_id = $data->user_id;
		$attendee->submitted_at = $data->submitted_at;
		$attendee->payment_hash = $data->payment_hash;
		$attendee->confirmed = $data->confirmed;
		$attendee->confirmed_at = $data->confirmed_at;

		return $attendee;
	}

	return false;
}

?>