<?php

add_action( 'init', 'camper_init_switch', 11 );

function camper_init_switch() {
	if ( isset( $_POST['camper-attend'] ) ) {
		camper_posted_attend();
	}
}

/* Attend */

add_shortcode( 'camper_attend', 'camper_shortcode_attend' );

function camper_shortcode_attend( $atts, $content = '' ) {
	global $current_user;

	if ( ! is_user_logged_in() ) {
		$html = '<p class="note">'
			. sprintf( __( "You must <a href=\"%s\">log in</a> to attend.", 'camper' ),
				wp_login_url( add_query_arg( array() ) ) ) . '</p>';

		return $html;
	}

	if ( ! $event = camper_event( $atts['event'] ) ) {
		$html = '<p class="error">' . esc_html( __( 'Event Not Found', 'camper' ) ) . '</p>';
		return $html;
	}

	if ( $event->closed ) {
		$html = '<p class="note">' . esc_html( __( "This event is closed.", 'camper' ) ) . '</p>';
		return $html;
	}

	$attendees = $event->attendees(
		array( 'user_id' => $current_user->ID ) );

	if ( $attendees ) {
		$html = '<p class="note">'
			. esc_html( __( "You have already applied for this event.", 'camper' ) ) . '</p>';

		$attendee = array_shift( $attendees );
		$item = camper_item( $attendee->item_id );

		$html .= '<ul class="items"><li>' . esc_html( $item->name ) . '</li></ul>';

		if ( ! $attendee->confirmed && $atts['paypal'] )
			$html .= camper_paypal_link( $item, $attendee, $atts );

		return $html;
	} else {
		return camper_attend_form( $event, $atts );
	}

}

function camper_attend_form( &$event, $atts = '' ) {
	$html = '<form method="post" action="">' . "\n";
	$html .= '<div class="attend">' . "\n";
	$html .= wp_nonce_field( 'camper-attend-' . $event->id, '_wpnonce', true, false );
	$html .= '<input type="hidden" name="event" value="' . esc_attr( $event->id ) . '" />';
	$html .= "\n";

	if ( $items = $event->items() ) {
		$html .= '<ul class="items">';
		foreach ( $items as $item ) {
			$html .= '<li><label><input type="radio" name="item" value="' . esc_attr( $item->id ) . '" /> ' . esc_html( $item->name ) . '</label></li>';
		}
		$html .= '</ul>';
	}

	$html .= '<input type="submit" name="camper-attend" value="' . esc_attr( __( 'Attend', 'camper' ) ) . '" />' . "\n";
	$html .= '</div>' . "\n";
	$html .= '</form>' . "\n";

	return $html;
}

function camper_paypal_link( &$item, &$attendee, $atts = '' ) {
	if ( ! $item )
		return;

	if ( $atts['use_sandbox'] )
		$action = 'https://www.sandbox.paypal.com/cgi-bin/webscr';
	else
		$action = 'https://www.paypal.com/cgi-bin/webscr';

	if ( ! $business = trim( $atts['paypal_business'] ) ) {
		if ( ( $event = camper_event( $item->event_id ) )
		&& ( $event_owner = get_userdata( $event->owner_id ) ) )
			$business = $event_owner->user_email;
	}

	if ( ! $item_name = trim( $atts['paypal_item_name'] ) )
		$item_name = $item->name;

	if ( $attendee->payment_hash )
		$item_name .= " [$attendee->payment_hash]";

	$notify_url = trim( $atts['paypal_notify_url'] );

	$html = '<form action="' . $action . '" method="POST">' . "\n";
	$html .= '<div class="camper paypal">' . "\n";
	$html .= '<input type="hidden" name="cmd" value="_xclick" />' . "\n";
	$html .= '<input type="hidden" name="upload" value="1" />' . "\n";

	$html .= '<input type="hidden" name="business" value="'
		. esc_attr( $business ) . '" />' . "\n";
	$html .= '<input type="hidden" name="item_name" value="'
		. esc_attr( $item_name ) . '" />' . "\n";

	$html .= '<input type="hidden" name="amount" value="' . $item->price . '" />' . "\n";
	$html .= '<input type="hidden" name="currency_code" value="' . $item->currency . '" />' . "\n";
	$html .= '<input type="hidden" name="rm" value="2" />' . "\n";
	$html .= '<input type="hidden" name="no_shipping" value="1" />' . "\n";
	$html .= '<input type="hidden" name="shipping" value="0" />' . "\n";
	$html .= '<input type="hidden" name="shipping2" value="0" />' . "\n";
	$html .= '<input type="hidden" name="tax" value="0" />' . "\n";

	$html .= '<input type="hidden" name="return" value="'
		. site_url( $_SERVER['REQUEST_URI'] ) . '" />' . "\n";
	$html .= '<input type="hidden" name="cancel_return" value="'
		. site_url( $_SERVER['REQUEST_URI'] ) . '" />' . "\n";
	$html .= '<input type="hidden" name="charset" value="utf-8" />' . "\n";

	if ( $notify_url )
		$html .= '<input type="hidden" name="notify_url" value="' . esc_url_raw( $notify_url ) . '" />' . "\n";

	if ( $attendee->payment_hash )
		$html .= '<input type="hidden" name="custom" value="' . esc_attr( $attendee->payment_hash ) . '" />' . "\n";

	$html .= '<input type="image" src="'
		. esc_url_raw( __( 'https://www.paypal.com/en_GB/i/btn/btn_paynow_LG.gif', 'camper' ) )
		. '" border="0" name="submit" alt="'
		. esc_attr( __( "PayPal - The safer, easier way to pay online.", 'camper' ) )
		. '"><img alt="" border="0" src="https://www.paypal.com/en_US/i/scr/pixel.gif" width="1" height="1">' . "\n";

	$html .= '</div>' . "\n";
	$html .= '</form>' . "\n";

	return $html;
}

function camper_posted_attend() {
	global $current_user;

	if ( ! is_user_logged_in() )
		return;

	if ( ! $event = camper_event( $_POST['event'] ) )
		return;

	if ( ! $item = camper_item( $_POST['item'] ) )
		return;

	if ( ! camper_check_referer( 'camper-attend-' . $event->id ) )
		return;

	$event->add_attendee( $item->id, $current_user->ID );

	wp_redirect( add_query_arg( array() ) );

	exit();
}

/* Attendee list */

add_shortcode( 'camper_attendees', 'camper_shortcode_attendees' );

function camper_shortcode_attendees( $atts, $content = '' ) {
	if ( ! $event = camper_event( $atts['event'] ) )
		return;

	$confirmed_only = (int) $atts['confirmed_only'];

	$attendees = $event->attendees( array( 'confirmed_only' => $confirmed_only ) );

	$html = '<div class="camper attendees">' . "\n";

	foreach ( (array) $attendees as $attendee ) {
		if ( ! $user = get_userdata( $attendee->user_id ) )
			continue;

		$html .= '<div class="attendee">';

		if ( $user->user_url )
			$html .= '<a href="' . esc_url_raw( $user->user_url ) . '" rel="nofollow">'
				. get_avatar( $user->ID, 42 ) . '</a>';
		else
			$html .= get_avatar( $user->ID, 42 );

		if ( $user->user_url )
			$html .= '<h4><a href="' . esc_url_raw( $user->user_url ) . '" rel="nofollow">'
				. $user->display_name . '</a></h4>';
		else
			$html .= '<h4>' . $user->display_name . '</h4>';

		if ( $company = get_usermeta( $user->ID, 'company' ) )
			$html .= '<div class="company">' . esc_html( trim( $company ) ) . '</div>';

		$html .= '</div>' . "\n";
	}

	$html .= '</div>' . "\n";

	return $html;
}

/* JavaScript & CSS */

if ( CAMPER_LOAD_JS )
	add_action( 'wp_print_scripts', 'camper_enqueue_scripts' );

function camper_enqueue_scripts() {
	$in_footer = true;
	if ( 'header' === CAMPER_LOAD_JS )
		$in_footer = false;

	wp_enqueue_script( 'camper', camper_plugin_url( 'scripts.js' ),
		array( 'jquery' ), CAMPER_VERSION, $in_footer );
}

if ( CAMPER_LOAD_CSS )
	add_action( 'wp_print_styles', 'camper_enqueue_styles' );

function camper_enqueue_styles() {
	wp_enqueue_style( 'camper', camper_plugin_url( 'styles.css' ),
		array(), CAMPER_VERSION, 'all' );
}

/* Misc */

function camper_check_referer( $action = -1, $query_arg = '_wpnonce' ) {
	return isset( $_REQUEST[$query_arg] )
		? wp_verify_nonce( $_REQUEST[$query_arg], $action ) : false;
}

?>